from __future__ import annotations
from typing import Any, Callable, Iterable


class DataTags:
    def __init__(self, **datatags: str | Iterable[str]) -> None:
        self._datatags: dict[str, str | Iterable[str]] = {}
        self._keys_value_pairs: set[tuple[str, str | Iterable[str]]] = set()

        if datatags:
            for key, value in datatags.items():
                self[key] = (
                    str(value)
                    if isinstance(value, str) or not isinstance(value, Iterable)
                    else tuple(sorted(value))
                )

    @property
    def all(self) -> frozenset[str]:
        return frozenset(self._datatags.keys())

    def __getitem__(self, key: str) -> str | Iterable[str]:
        return self._datatags[key]

    def __contains__(self, key: str) -> bool:
        return key in self._datatags

    def __setitem__(self, key: str, value: str | Iterable[str]) -> None:
        self._keys_value_pairs.add(
            (key, value if isinstance(value, str) else tuple(value))
        )
        self._datatags[key] = value

    def __delitem__(self, key: str) -> None:
        if key not in self._datatags:
            return
        self._keys_value_pairs.remove((key, self._datatags[key]))
        del self._datatags[key]

    def __hash__(self) -> int:
        return hash((tuple(self._keys_value_pairs)))

    def __eq__(self, __value: object) -> bool:
        return self._datatags == __value


def filter_datatags_by_value(
    key: str,
    value: str,
    datatags_objects: Iterable[Any],
    datatags_retriever: Callable[[Any], DataTags],
):
    for datatags_object in datatags_objects:
        datatags = datatags_retriever(datatags_object)
        if datatags[key] == value:
            yield datatags_object
