from enum import Enum
from avariantas.information.datatags import DataTags
from avariantas.problems.exceptions import InvalidFeaturePositionException


class KnownTags(Enum):
    POSITION = 0
    LENGTH = 1


class Annotation:
    def __init__(self, position: int, length: int, datatags: DataTags) -> None:
        if position < 0:
            raise InvalidFeaturePositionException(
                f"A feature cannot have a position that is less than 0 "
                f"({position} < 0)",
            )
        if length < 0:
            raise InvalidFeaturePositionException(
                f"A feature cannot have a length that is less than 0 (length is "
                f"{length} < 0)",
            )
        self._datatags = datatags
        self.datatags[KnownTags.POSITION.name] = str(position)
        self.datatags[KnownTags.LENGTH.name] = str(length)

    @property
    def datatags(self):
        return self._datatags

    @property
    def position(self):
        return int(str(self.datatags[KnownTags.POSITION.name]))

    @property
    def length(self):
        return len(self)

    @property
    def start(self):
        return self.position

    @property
    def end(self):
        return self.position + self.length

    def __len__(self):
        return int(str(self.datatags[KnownTags.LENGTH.name]))
