from typing import Iterable


class IllegalStateChangeException(Exception):
    pass


class IngressException(Exception):
    pass


class InvalidDatatagsException(IngressException):
    pass


class InvalidFeaturePositionException(Exception):
    pass


class RepresentationException(Exception):
    pass


class RemoteSearchException(IngressException):
    pass


class OrganizedMutationException(IllegalStateChangeException):
    pass


class GroupNameException(Exception):
    pass


class GroupNameParsingException(GroupNameException):
    pass


class InvalidFileException(IngressException):
    pass


class AssociationTestFailedException(Exception):
    pass


class ExceptionSeries(Exception):
    def __init__(
        self, exceptions: Iterable[Exception], name: str, *args: object
    ) -> None:
        self._series_name = name
        self._exceptions = tuple(exceptions)
        super().__init__(*args)

    @property
    def exceptions(self):
        return tuple(self._exceptions)

    def __str__(self) -> str:
        builder = super().__str__()
        builder = f'\n"{self._series_name}" Series of exceptions:\n'
        builder += "\n"
        for exception in self._exceptions:
            builder += str(exception)
            builder += "\n"
        return builder
