from __future__ import annotations

from argparse import _SubParsersAction, ArgumentParser
from enum import Enum
from typing import Iterable
from avariantas.egress.local import save_project
from avariantas.information.mutations import group_name_from_query

from avariantas.ingress.local import load_project


class MutationOperations(Enum):
    UNION = 0
    INTERSECT = 1
    BUILD = 2


def build_mutation_group(
    project_path: str, operation_name: str, group_names: Iterable[str], alias: str
):
    project = load_project(project_path)
    organized_mutations = project.organized_mutations
    operation = MutationOperations[operation_name.upper()]
    if operation == MutationOperations.BUILD:
        for group_name in group_names:
            organized_mutations.build_group_from_name(
                group_name_from_query(
                    group_name, alias=alias if len(alias) == 0 else None
                )
            )
    elif operation == MutationOperations.UNION:
        for group_name in group_names:
            organized_mutations.union(
                group_name_from_query(
                    group_name, alias=alias if len(alias) == 0 else None
                )
            )
    elif operation == MutationOperations.INTERSECT:
        for group_name in group_names:
            organized_mutations.intersect(
                group_name_from_query(
                    group_name, alias=alias if len(alias) == 0 else None
                )
            )
    save_project(project, project_path)


def add_mutation_parser(subparsers: _SubParsersAction[ArgumentParser]):
    parser = subparsers.add_parser(
        "mutations", help="Modify groupings of mutations by performing set operations."
    )

    parser.add_argument(
        "operation",
        type=str,
        choices=[str(val.name).lower() for val in MutationOperations],
        help="The operation to perform on the groups of mutations",
        nargs=1,
        metavar="N",
    )

    parser.add_argument(
        "group_names",
        type=str,
        help="The group names to operate using.",
        nargs="+",
        metavar="G",
    )
    parser.add_argument(
        "-a",
        "--alias",
        type=str,
        help="An alias for the group that will be made.",
        required=False,
        default=None,
    )
