from __future__ import annotations
from argparse import ArgumentParser
import logging

from avariantas.ui.cli.association import (
    add_associations_parser,
    perform_association_test,
)
from avariantas.ui.cli.mutations import add_mutation_parser, build_mutation_group
from avariantas.ui.cli.project import add_project_parser, manage_project

cli_logger = logging.getLogger(__name__)


def run(args):
    action = args.action
    if action == "project":
        manage_project(
            args.operation,
            args.project_path,
            args.vcfs,
            args.recursive,
            args.additional_info,
            args.additional_info_header,
            args.search_1kgenome,
            args.population_code,
        )
    elif action == "association":
        perform_association_test(
            args.project_path,
            args.tag_a,
            args.tag_b,
            args.count_individuals,
            args.group_a,
            args.group_b,
            args.test_type,
            args.repeat_for,
        )
    elif action == "mutations":
        build_mutation_group(
            args.project_path, args.operation, args.group_names, args.alias
        )


def main():
    parser = ArgumentParser(
        prog="avariantas",
        description="Avariantas is an Automatic Variant Association Statistics tool "
        "capable of performing various statistical tests on large VCF datasets with "
        "consideration to many variables. Use as a python library to use the full "
        "potential of this program.",
    )

    parser.add_argument(
        "-V",
        "--verbosity",
        type=str,
        help="The verbosity of the logging.",
        default="INFO",
        required=False,
    )

    parser.add_argument(
        "project_path",
        type=str,
        help="Designates the project path to work on.",
        metavar="P",
    )
    console_log_format = logging.Formatter(
        "[%(levelname)s - %(asctime)s - %(name)s] %(message)s"
    )
    subparsers = parser.add_subparsers(dest="action", required=True)
    add_project_parser(subparsers)
    add_mutation_parser(subparsers)
    add_associations_parser(subparsers)
    root_logger = logging.getLogger()
    console_handle = logging.StreamHandler()
    console_handle.setFormatter(console_log_format)
    root_logger.addHandler(console_handle)
    args = parser.parse_args()
    root_logger.setLevel(args.verbosity.upper())
    run(args)


if __name__ == "__main__":
    main()
